@@ -50,10 +50,7 @@ class BackwardEulerModel(ChasteModel):
                    self._model.state_vars), key=lambda s: get_variable_name(s, s in self._in_interface))
         # Pick the formatted equations that are for non-linear derivatives
 
-        non_linear_derivs = (eq.lhs for eq in self._derivative_equations if isinstance(eq.lhs, Derivative)
-                             and eq.lhs.args[0] in self._non_linear_state_vars)
-
-        self._non_linear_eqs = tuple(e for e in self._model.get_equations_for(non_linear_derivs))
+        self._non_linear_eqs = self._get_non_linear_eqs()
 
         self._jacobian_equations, self._jacobian_matrix = \
             get_jacobian(self._non_linear_state_vars,
@@ -62,6 +59,13 @@ class BackwardEulerModel(ChasteModel):
         self._linear_deriv_eqs, self._linear_equations, self._vars_in_one_step = \
             self._rearrange_linear_derivs()
 
+    def _get_non_linear_eqs(self):
+        """Get derivative eqs for non linear state vars"""
+        non_linear_derivs = (eq.lhs for eq in self._derivative_equations if eq.lhs in self._model.y_derivatives
+                             and eq.lhs.args[0] in self._non_linear_state_vars)
+        non_linear_deriv_eqs = tuple(e.lhs for e in self._model.get_equations_for(non_linear_derivs))
+        return tuple(eq for eq in self._derivative_equations if eq.lhs in non_linear_deriv_eqs)
+
     def _rearrange_linear_derivs(self):
         """Formats the rearranged linear derivative expressions
 
